@@ -22,7 +22,7 @@ def findall(current_node: typing.Union[dict, list], seeked_xpath_str: str, raise
     ]
     return _findall(current_node, seeked_xpath_list)
 # ******************************************************************************
-def findsingle(current_node: typing.Union[dict, list], seeked_xpath_str: str, raise_exception = True) -> tuple:
+def findfirst(current_node: typing.Union[dict, list], seeked_xpath_str: str, raise_exception = True) -> tuple:
     found = findall(current_node, seeked_xpath_str, False)
     if not found:
         if raise_exception:
@@ -324,7 +324,7 @@ def _findall(
             multi_found = {}
             # ******************************************************************
             # just skip * and check if the current node is satisfied to the xpath/conditions
-            n0print("*"*30 + " NOT TESTED #2...")
+            ## n0print("*"*30 + " NOT TESTED #2...")
             found = _findall(
                             parent_node,
                             seeked_xpath_list[1:],
@@ -340,12 +340,16 @@ def _findall(
             for child_name in parent_node: # checking * sub-nodes
                 child_node = parent_node[child_name]
                 if isinstance(child_node, (dict, list)):
-                    n0print("*"*30 + " NOT TESTED #3...")
+                    ## n0print("*"*30 + " NOT TESTED #3...")
+                    # n0debug("child_node")
+                    n0debug("seeked_xpath_list")
+                    n0debug_calc(found_xpath_list + [child_name], "found_xpath_list + [child_name]") ## n0debug_calc(found_xpath_list + seeked_xpath_list[0:1], "found_xpath_list + seeked_xpath_list[0:1]")
+                    n0debug_calc("/".join(found_xpath_list).replace('/[', '['), "parent_node")
                     found = _findall(
                                     child_node,
                                     seeked_xpath_list,
                                     # found_xpath_list + [child_name],
-                                    found_xpath_list + seeked_xpath_list[0:1],
+                                    found_xpath_list + [child_name], ##found_xpath_list + seeked_xpath_list[0:1],
                                     # parent_nodes_stack + [(found_xpath_list, parent_node)],
                                     {**parent_nodes_stack, **{"//" + "/".join(found_xpath_list).replace('/[', '['): parent_node}},
                                     raise_exception,
